var stringsvc = angular.module('strings-module', []);

stringsvc.service("stringsvc", function () {

    this.compareVersions = function (versionA, versionB) {
        // old version format: "10.3.0.0.20180801.CentOS6"
        // version format: "10.3.0-1.201809260103.CentOS6"
        // {major}.{minor}.{patch}-{buildNumber}.{date}.{platform}
        // Return 1  if a > b
        // Return -1 if a < b
        // Return 0  if a == b

        if (versionA === versionB) {
            return 0;
        }

        var versionAParts = versionA.split('-');
        var releaseA = versionAParts[0];
        var buildA = versionAParts[1] ? versionAParts[1].split('.')[0] : versionA.split('.')[3];

        var versionBParts = versionB.split('-');
        var releaseB = versionBParts[0];
        var buildB = versionBParts[1] ? versionBParts[1].split('.')[0] : versionB.split('.')[3];

        // Releases do not equal, check them first
        if (releaseA !== releaseB) {
            var releaseAParts = releaseA.split('.');
            var releaseBParts = releaseB.split('.');

            // handle previous version formats that may not have included a dash
            var maxPartsLength = releaseAParts.length < 3 ? releaseAParts : 3;
            for (var i = 0; i < maxPartsLength; i++) {
                if (releaseAParts[i] === releaseBParts[i]) {
                    continue;
                }
                if (parseInt(releaseAParts[i]) < parseInt(releaseBParts[i])) {
                    return -1;
                }
                if (parseInt(releaseAParts[i]) > parseInt(releaseBParts[i])) {
                    return 1;
                }
            }
        }

        // Releases are equal, so we compare the build number ignoring the date and platform and taking old format into account
        var buildANumber = parseInt(buildA);
        var buildBNumber = parseInt(buildB);
        return buildANumber === buildBNumber ? 0 : buildANumber - buildBNumber;
    }
});
