//(function($, PHD, window, undefined) {

function processProtectPage(PHD, window, $scope, $rootScope, $compile, $analytics, inventorysvc, gettextCatalog, uiGridConstants, localStorageService) {
    var tabs = [],
        initialTab = $.cookie("initial_tab_protect") || 0,
        currentTabIndex = 0,
        $protectTreeContainer = $("#protect-treeview"),
        $copyTreeContainer = $("#copy-treeview"),
        TabTimer, UserPreference,
        slugs;

    var TAB_BAR_HEIGHT = 42;
    var MIN_VIRTUAL_THRESHOLD = 30;

    if ($rootScope.isTarget) {
        slugs = ['inventory', 'copies'];
    } else {
        slugs = ['inventory'];
    }

    // lmc - add this.
    var refreshRate = refreshRates.page_protect_refresh;
    $rootScope.ProtectTimer.refreshControl = $("#protect-page-preferences .timer-refresh");
    TabTimer = $rootScope.ProtectTimer;

    var LARGE_INVENTORY = 2500;

    // template for the weekly status report that produces 1 status icon per day based on backup status.
    // the grid allows selection for all columns, disable by stopping event propagation so that clicks bring up status details.
    var p_backupColoredStatusTemplate = '<div class="ui-grid-cell-contents" ng-click="$event.stopPropagation()"> ' +
        '<i ng-repeat="a in row.entity.last_backups track by $index" ' +
        'ng-click="row.entity.last_backups[$index].disabled || grid.appScope.showStatusDetails(row.entity, 0, $index, $event)" ' +
        'class="{{grid.appScope.showBackupStatus(row.entity.last_backups, $index)}} report-status-icon" ' +
        'title="{{grid.appScope.showDayTitle($index)}}"></i>' +
        '</div>';

    // template for the weekly status report that produces 1 status icon per day based on backup copy status.
    // the grid allows selection for all columns, disable by stopping event propagation so that clicks bring up status details.
    var p_backupCopyColoredStatusTemplate = '<div class="ui-grid-cell-contents" ng-click="$event.stopPropagation()"> ' +
        '<i ng-repeat="a in row.entity.last_backup_copies track by $index" ' +
        'ng-click="row.entity.last_backup_copies[$index].disabled || grid.appScope.showStatusDetails(row.entity, 1, $index, $event)" ' +
        'class="{{grid.appScope.showBackupStatus(row.entity.last_backup_copies, $index)}} report-status-icon" ' +
        'title="{{grid.appScope.showDayTitle($index)}}"></i>' +
        '</div>';

    var SEARCH_ALL = 1;
    var SEARCH_SUCCESSES = 2;
    var SEARCH_NOSUCCESSES = 3;
    var SEARCH_FAIL_TODAY = 4;
    var SEARCH_WARNING_TODAY = 5;
    var SEARCH_FAILURES = 6;
    var SEARCH_WARNINGS = 7;
    var SEARCH_ANY = 8;

    // Options for the filter dropdowns for the weekly status report.
    $scope.buildFilterOptions = function(type) {
        var anyString = type === 'last_backups' ? gettextCatalog.getString('Any backups in last 7 days') : gettextCatalog.getString('Any copies in last 7 days');
        return [
            { value: SEARCH_ALL, label: gettextCatalog.getString('All') },
            { value: SEARCH_SUCCESSES, label: gettextCatalog.getString('Only Successes in last 7 days') },
            { value: SEARCH_NOSUCCESSES, label: gettextCatalog.getString('No Successes in last 7 days') },
            { value: SEARCH_FAIL_TODAY, label: gettextCatalog.getString('Failed Today') },
            { value: SEARCH_WARNING_TODAY, label: gettextCatalog.getString('Warning Today') },
            { value: SEARCH_FAILURES, label: gettextCatalog.getString('Failed in last 7 days') },
            { value: SEARCH_WARNINGS, label: gettextCatalog.getString('Warning in last 7 days') },
            { value: SEARCH_ANY, label: anyString }
        ];
    };

    // Save day of week for search and status drill-down.
    $scope.dayOfWeek = moment().day();

    /*
    * Functions for the weekly status report.
    */

    // Returns a formatted Day and date, e.g., Sunday, June 13, 2016.
    var dow_weekdays = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
    function getDay(day, dow) {
        var offset = day - dow;
        if (dow < day) {
            offset -= 7;    // go to prior week.
        }
        var date = moment().day(dow + offset);
        return dow_weekdays[day] + ", " + date.format(DATE_DISPLAY_FORMAT);
    }

    // Returns a status icon for the day, or "none" if on backups (or copies) are available.
    // Sets disabled to false so that the click function will not open a details dialog.
    $scope.showBackupStatus = function(element, day) {
        var obj = element[day];
        var status = "none";
        if (obj !== undefined) {
            if (obj.successes !== undefined && obj.successes.count > 0) {
                status = 'success';
            }
            if (obj.warnings !== undefined && obj.warnings.count > 0) {
                status = 'warning';
            }
            if (obj.failures !== undefined && obj.failures.count > 0) {
                status = 'error';
            }
            // See if the element should be clickable or not.
            // If the object property is disabled, the ng-click function in the cell template will not open a dialog.
            if (status !== "none") {
                status += ' day-result'; // so that it is clickable.
            } else {
                obj.disabled = true;
            }
        }
        return "icon-phd-" + status;
    };

    // Given the chosen filter (searchTerm), decide which rows should be seen (status of true), or removed from view (status of false).
    function selectStatusFilterFunction (searchTerm, cellValue) {
        if (searchTerm === SEARCH_ALL) {
            return true;
        }
        var status = searchTerm === SEARCH_NOSUCCESSES;
        if (cellValue instanceof Array) {
            for (var i = 0; i < cellValue.length; i++) {
                var day = cellValue[i];
                if (searchTerm === SEARCH_SUCCESSES) {
                    if (status === false && day.successes !== undefined && day.successes.count > 0) {
                        status = true;
                    }
                    if ((day.warnings !== undefined && day.warnings.count > 0) || day.failures !== undefined && day.failures.count > 0) {
                        status = false;
                        break;
                    }
                } else if (searchTerm === SEARCH_FAILURES) {
                    if (day.failures !== undefined && day.failures.count > 0) {
                        status = true;
                        break;
                    }
                } else if (searchTerm === SEARCH_WARNINGS) {
                    if (day.warnings !== undefined && day.warnings.count > 0) {
                        status = true;
                        break;
                    }
                } else if (searchTerm === SEARCH_FAIL_TODAY) {
                    if (day.failures !== undefined && day.failures.count > 0 && day.day === $scope.dayOfWeek) {
                        status = true;
                        break;
                    }
                } else if (searchTerm === SEARCH_WARNING_TODAY) {
                    if (day.warnings !== undefined && day.warnings.count > 0 && day.day === $scope.dayOfWeek) {
                        status = true;
                        break;
                    }
                } else if (searchTerm === SEARCH_NOSUCCESSES) {
                    if (day.successes !== undefined && day.successes.count > 0) {
                        status = false;
                        break;
                    }
                } else if (searchTerm === SEARCH_ANY) {
                    if (day.successes !== undefined && day.successes.count > 0 ||
                        day.warnings !== undefined && day.warnings.count > 0 ||
                        day.failures !== undefined && day.failures.count > 0) {
                        status = true;
                        break;
                    }
                }
            }
        }
        return status;
    }

    var nameCol = {
        field: "name",
        headerCellFilter: 'translate',
        cellTooltip: true,
        displayName: gettextCatalog.getString("Name"),
        filter: {placeholder:  gettextCatalog.getString("Name")},
        sortable : true,
        sort: {
            direction: 'asc',
            priority: 0
        }
    };

    var backupsCol = {
        field: "last_backups",
        headerCellFilter: 'translate',
        //headerCellTemplate: backupsHeaderTemplate,
        enableSorting: false,
        cellTemplate: p_backupColoredStatusTemplate,
        displayName: gettextCatalog.getString("Backups"),
        headerCellClass: 'weekly-header-class',
        filter: {
            term: SEARCH_ALL,
            selectOptions: $scope.buildFilterOptions('last_backups'),
            type: uiGridConstants.filter.SELECT,
            condition: selectStatusFilterFunction
        },
        width: 220
    };

    var copiesCol = {
        field: "last_backup_copies",
        headerCellFilter: 'translate',
        //headerCellTemplate: backupCopiesHeaderTemplate,
        enableSorting: false,
        cellTemplate: p_backupCopyColoredStatusTemplate,
        displayName: gettextCatalog.getString("Backup Copies"),
        headerCellClass: 'weekly-header-class',
        filter: {
            term: SEARCH_ALL,
            selectOptions: $scope.buildFilterOptions('last_backup_copies'),
            type: uiGridConstants.filter.SELECT,
            condition: selectStatusFilterFunction
        },
        width: 220
    };

    $scope.protectedAssetsColumnDefs = [
        nameCol,
        backupsCol,
        copiesCol
    ];

    $scope.copiedAssetsColumnDefs = [
        jQuery.extend(true, {}, nameCol),
        jQuery.extend(true, {}, copiesCol)
    ];

    $scope.gridList = [0, 0];
    $scope.initialColumnOrder = [[], []];
    $scope.columnOrderRestored = false;
    $scope.placeHolder = gettext("There are no assets to display");

    $scope.assetsGrid ={
        //columnDefs:
        //data: 'jobsList',
        rowHeight: 30,
        enableHorizontalScrollbar:0,
        //enableVerticalScrollbar:0,
        enableGridMenu: true,
        enableFiltering: true,
        resizable: true,
        multiSelect: true,
        enableFullRowSelection : true,      // click on name to select, disabled in cellTemplate for backup and copy status.
        enableRowHeaderSelection: true,
        selectionRowHeaderWidth: 16,
        gridMenuShowHideColumns: true,
        enableColumnMenus: true,
        gridMenuCustomItems: [{
            title: gettextCatalog.getString('Reset column defaults'),
            action: function($event) {
                // clear the current grids, then re-draw as the original.
                $scope.restoreGridState();
            },
            order: 99
        }],
        gridMenuTitleFilter: function(title) {
            return gettextCatalog.getString(title);
        },
        onRegisterApi: function (gridApi) {
            $scope.onRegisterGrid(gridApi);
        }
    };

    // Two states, one for each tab.
    $scope.savedState = [undefined, undefined];

    $scope.onRegisterGrid = function(gridApi) {
        var tabIndex = gridApi.grid.options.tabIndex;
        $scope.gridApi = gridApi;
        $scope.gridList[tabIndex] = gridApi;
        gridApi.core.on.rowsRendered($scope, function() {
            var tabIndex = $scope.gridApi.grid.options.tabIndex;
            if (!$scope.stateExists(tabIndex)) {
                $scope.saveGridState(tabIndex);
                $scope.restorePersistedState($scope.gridApi, tabIndex);
            }
        });
        // configure handler for row selection changes
        gridApi.selection.on.rowSelectionChanged($scope, $scope.onSelectionChanged);
        gridApi.selection.on.rowSelectionChangedBatch($scope, $scope.onSelectionChanged);

        // configure handler to persist grid state changes
        gridApi.colMovable.on.columnPositionChanged($scope, $scope.persistGridState);
        gridApi.colResizable.on.columnSizeChanged($scope, $scope.persistGridState);
        gridApi.core.on.columnVisibilityChanged($scope, $scope.persistGridState);
        gridApi.core.on.filterChanged($scope, $scope.filterChanged);
        gridApi.core.on.sortChanged($scope, $scope.persistGridState);
    };

    /*
    * If filter cleared, set term on certain columns to 1 (ALL).
    */
    $scope.filterChanged = function() {

        var grid = this.grid;
        checkAndSetColumn(grid.getColumn('last_backups'));
        checkAndSetColumn(grid.getColumn('last_backup_copies'));

        // If the filter SELECT columns are not set, set to default, 'All', value 1.
        function checkAndSetColumn(column) {
            if (column !== null && column.filters !== undefined) {
                if (column.filters[0].term === undefined || column.filters[0].term === null) {
                    column.filters[0].term = SEARCH_ALL;
                }
            }
        }
        $scope.persistGridState();

    };

    /*
    * Returns true if a cached state exists for the job tab, false if not.
    * Checks for the existence of columns because these may be delayed (returned after API returns).
    */
    $scope.stateExists = function(tabIndex) {
        return ($scope.savedState[tabIndex] !== undefined && $scope.savedState[tabIndex].columns.length > 0);
    };

    /*
    * Use ui-grid state to save information in an array about the original grid: column order, width, filters, etc.
    */
    $scope.saveGridState = function(tabIndex) {
        $scope.savedState[tabIndex] = $scope.gridApi.saveState.save();
    };

    /*
    * Save state of the grid in local storage: job-tab-N, where the state is defined by the UI-grid.
    */
    $scope.persistGridState = function() {
        var tabIndex = $scope.gridApi.grid.options.tabIndex;
        var token = 'protect-tab-' + tabIndex;
        localStorageService.set(token, $scope.gridApi.saveState.save());
    };

    /*
    * Restore the grid's state from the local storage for this tab, if it exists.
    */
    $scope.restorePersistedState = function(gridApi, tabIndex) {
        if (!$scope.columnOrderRestored && gridHasColumns(gridApi)) {
            var token = 'protect-tab-' + tabIndex;
            var jobState = localStorageService.get(token);
            if (jobState !== null && jobState !== undefined) {
                gridApi.saveState.restore($scope, jobState);
                $scope.columnOrderRestored = true;
            }
        }
    };

    // Returns true if the grid exists and its columns are present, and false otherwise.
    // The ui-grid will first return the grid structure, with the column population delayed.
    var gridHasColumns = function(gridApi) {
        return gridApi !== null && gridApi.grid.columns !== undefined && gridApi.grid.columns.length > 0;
    };

    /*
    * Restore this grid's state from the array (the original tabs), and delete the local storage.
    */
    $scope.restoreGridState = function() {
        var tabIndex = $scope.gridApi.grid.options.tabIndex;
        $scope.gridApi.saveState.restore($scope, $scope.savedState[tabIndex]);
        setTimeout(function () {
            var token = 'protect-tab-' + tabIndex;
            localStorageService.remove(token);
        }, 100);
        $scope.columnOrderRestored = false;
    };

    $scope.onSelectionChanged = function(rows) {
        var gridApi = $scope.gridApi;
        //console.log("selection in tab: " + $scope.gridApi.grid.options.tabIndex);
        var self = tabs[$scope.gridApi.grid.options.tabIndex];
        var selectedItems = gridApi.selection.getSelectedRows();
        if (selectedItems.length >= 1) {
            var sameRow = (selectedItems.length === 1 && self.selectedID === selectedItems[0].id);
            self.selectedID = selectedItems[0].id;
            self.rowData = selectedItems[0];
            self.$selected = selectedItems;
            var Ids = [];
            for (var i = 0; i < selectedItems.length; i++) {
                Ids.push( { 'id' : selectedItems[i].id, 'enabled' : selectedItems[i].enabled, 'type': selectedItems[i].type });
            }
            self.selectedIDs = Ids;
            if (selectedItems.length > 1) {
                $scope.firstSelectedID = getFirstSelected($scope.gridApi.grid, selectedItems);
            } else {
                $scope.firstSelectedID = selectedItems[0].id;
            }
            setTimeout(function () {
                $scope.$apply();
            }, 100);
            self.updateButtons();
        } else if (selectedItems.length === 0) {
            self.selectedID = null;
            self.rowData = null;
            self.$selected = [];
            self.selectedIDs = [];
            self.updateButtons();
        }
    };

    function getFirstSelected(grid, selectedItems) {
        var rowIndex = grid.renderContainers.body.visibleRowCache.length - 1;
        var firstSelected = null;
        for (var i = 0; i < selectedItems.length; i++) {
            var item = selectedItems[i];
            for (var j = 0; j < grid.renderContainers.body.visibleRowCache.length && rowIndex > 0; j++) {
                if (item.id === grid.renderContainers.body.visibleRowCache[j].entity.id) {
                    if (j < rowIndex) {
                        rowIndex = j;
                        firstSelected = item.id;
                    }
                    break;
                }
            }
        }
        return firstSelected;
    }

    /*
    * If the user clicks on the status icon, this function creates the dialog with summary information.
    */
    $scope.showStatusDetails = function(row, type, day, event) {
        var tab = tabs[$scope.gridApi.grid.options.tabIndex];
        var self = tab !== undefined && tab.tabs !== undefined ? tab.tabs[0] : {'isCopyTab': false};

        var element = type === 0 ? row.last_backups : row.last_backup_copies;
        $scope.dayStatus = element[day];

        // Get system ID from the node, or set to local system if on copy tab.
        if (self.isCopyTab) {
            $scope.dayStatus.sid = $rootScope.local.id;
        } else {
            if (row.id !== undefined) {
                var a = row.id.split('_');
                $scope.dayStatus.sid = a[0];
            } else {
                $scope.dayStatus.sid = -1;
            }
        }
        $scope.dayStatus.name = row.name;
        $scope.dayStatus.dayString = getDay(day, $scope.dayOfWeek);
        $scope.dayStatus.type = type === 0 ? "Backups" : "Backup Copies";

        var minString = (type === 0) ? "260" : "300";
        var dayStatusDialogStyle = "width:auto; height:auto; min-width:" + minString + "px; min-height:240px";
        var targetsArray = [];

        // Format backup copy targets
        var targets = $scope.dayStatus.backup_copy_targets;
        if (targets !== undefined) {
            for (var targetName in targets) {
                if (targets.hasOwnProperty(targetName)) {
                    var obj = targets[targetName];
                    obj.name = targetName;
                    var targetIDs = obj.successes.ids.concat(obj.failures.ids, obj.incomplete.ids);
                    obj.ids = targetIDs.toString();
                    targetsArray.push(obj);
                }
            }
        }
        $scope.dayStatus.targets = targetsArray;

        if ($scope.dayStatus.warnings === undefined) {
            $scope.dayStatus.warnings = { 'ids': [] };
        }
        var all_ids = $scope.dayStatus.successes.ids.concat($scope.dayStatus.failures.ids, $scope.dayStatus.warnings.ids, $scope.dayStatus.incomplete.ids);
        $scope.dayStatus.all_ids = all_ids.toString();

        $scope.dayStatus.formatted_day = $scope.dayStatus.dayString;
        $scope.dayStatus.formatted_name = $scope.dayStatus.name.replace(/'/g, "\\'");
        $scope.dayStatus.override_copies = self.isCopyTab ? 1 : 0;

        var html = $scope.dayStatus.targets.length === 0 ? dayTmpl($scope.dayStatus) : targetsTmpl($scope.dayStatus);
        var $anchor = $(event.target);
        html = $compile(html)($scope);
        $("<div/>").html(html).popover({
            anchor: $anchor,
            autoOpen: true,
            popoverClass: 'job-inventory-result',
            destroyOnClose: true
        });
    };

    $scope.protectedAssetsGrid = jQuery.extend(true, {}, $scope.assetsGrid);
    $scope.protectedAssetsGrid.columnDefs = $scope.protectedAssetsColumnDefs;
    $scope.protectedAssetsGrid.data = 'protectedAssetsStatus';
    $scope.protectedAssetsGrid.tabIndex = 0;

    $scope.copiedAssetsGrid = jQuery.extend(true, {}, $scope.assetsGrid);
    $scope.copiedAssetsGrid.columnDefs = $scope.copiedAssetsColumnDefs;
    $scope.copiedAssetsGrid.data = 'copiedAssetsStatus';
    $scope.copiedAssetsGrid.tabIndex = 1;


    // Rebuild options on language change.
    $scope.$on("languageChanged", function (event){
        $scope.protectedAssetsGrid.columnDefs[1].filter.selectOptions = $scope.buildFilterOptions('last_backups');
        $scope.protectedAssetsGrid.columnDefs[2].filter.selectOptions = $scope.buildFilterOptions('last_backup_copies');
        $scope.copiedAssetsGrid.columnDefs[1].filter.selectOptions = $scope.buildFilterOptions('last_backup_copies');

        for (var i = 0; i < $scope.gridList.length; i++) {
            var token = 'protect-tab-' + i;
            setTimeout(function (token) {
                localStorageService.remove(token);
            }, 100, token);
            $scope.columnOrderRestored = false;
        }
    });

    $scope.getRows = function(tabIndex) {
        var rows = null;
        switch (tabIndex) {
            case 0:
                rows = $scope.protectedAssetsStatus;
                break;
            case 1:
                rows = $scope.copiedAssetsStatus;
                break;
        }
        return rows;
    };

    $scope.selectGridRowsByIDs = function(ids, tabIndex) {
        var gridApi = $scope.gridList[tabIndex];
        gridApi.grid.modifyRows($scope.getRows(tabIndex)); //$scope.jobsList);
        setTimeout(function() {
            var rows = $scope.getRows(tabIndex);
            for (var i = 0; i < rows.length; i++) {
                var row = rows[i];
                for (var j = 0; j < ids.length; j++) {
                    if (row.id === ids[j].id) {
                        gridApi.selection.selectRow(row);
                        break;
                    }
                }
            }
        }, 0);
    };

    $scope.updateGridData = function(tabIndex, data) {
        var gridToShow = null;
        switch (tabIndex) {
            case 0:
                $scope.protectedAssetsStatus = data;
                gridToShow = $scope.protectedAssetsGrid;
                break;
            case 1:
                $scope.copiedAssetsStatus = data;
                gridToShow = $scope.copiedAssetsGrid;
                break;
        }
        if (gridToShow && (currentTabIndex === tabIndex)) {
            if ($scope.gridList[tabIndex] !== 0) {
                $scope.gridApi = $scope.gridList[tabIndex];
            }
            // reset select All on update.
            $scope.gridApi.grid.selection.selectAll = false;
            //console.log("--->jobs update for tab: " + $scope.gridApi.grid.options.tabIndex);
            gridToShow.minRowsToShow = data.length;
            $scope.gridApi.grid.gridHeight = $scope.setGridHeight(tabIndex, false);
            var threshold = $scope.gridApi.grid.gridHeight / gridToShow.rowHeight;
            if (threshold > MIN_VIRTUAL_THRESHOLD) {
                gridToShow.virtualizationThreshold = threshold;
            } else {
                gridToShow.virtualizationThreshold = MIN_VIRTUAL_THRESHOLD;
            }
        } else {
            console.log("--->jobs data skipped, tab: " + currentTabIndex + ": not: "+ tabIndex);
        }
    };

    $scope.getAvailableVerticalSpace = function($element, fullHeightContainer) {
        var $parent = $element.parent(),
            $container = $element.closest(fullHeightContainer);

        if (!$container.length) {
            return $parent.outerHeight();
        } else {
            return $container.outerHeight();
        }
    };

    $scope.setGridHeight = function(tabIndex, detailsOpen) {
        tabs[tabIndex].fullHeight = $scope.getAvailableVerticalSpace(tabs[tabIndex].$buttons, ".full-height") - TAB_BAR_HEIGHT;
        var gridHeight = tabs[tabIndex].fullHeight;
        if (detailsOpen) {
            gridHeight -= tabs[tabIndex].inventoryTableHeight;
        }
        $scope.setGridElementHeight($('.ui-grid-full-height'), gridHeight);
        //console.log(gridHeight);
        return gridHeight;
    };

    $scope.setGridElementHeight = function($element, height) {
        $element.height(height);
    };

    function setPageRefresh(interval) {
        refreshRate = interval;
        tabs.forEach(function(tab) {
            if ( _.isFunction(tab.setTimerInterval)) {
                tab.setTimerInterval(refreshRate);
            }
        });
    }

    function initTabs() {

        if (!$scope.user.loggedIn) {
            TabTimer.rerun();
            return;
        }

        $(".page-tab").each(function (index) {
            if (index === 0) {
                tabs[index] = new ProtectInventoryTab(index, $(this), slugs[index]).init();
            } else {
                tabs[index] = new CopyInventoryTab(index, $(this), slugs[index]).init();
            }
        });
        tabs[currentTabIndex].activate();
    }

    var ProtectInventoryTab = function (index, panel, slug) {
        this.index = index;
        this.$panel = panel;
        this.namespace = "protect" + slug;

        this.$buttons = this.$panel.find(".button-bar");
        this.$buttons.find(".btn-group-recover").flyout();
        this.$buttons.find(".replicas-btn-group").flyout();

        this.$tabs = $('#protect-inventory-details');
        this.tabs = [];
        this.slugs = ['status'];

        this.TreeViewData = null;

        this.navGroups = null;

        this.inventoryTree = PHD.ProtectTreeView(
            $protectTreeContainer,
            $("#protect-treeview-selections")
        );

        this.inventoryTreeHeader = PHD.TreeViewHeader(
            $("#protect-treeview-header"),
            this.inventoryTree,
            "protect"
        );

        this.inventoryTreeSearch = null;
        this.currentTabIndex = 0;
        this.active = false;
    };

    ProtectInventoryTab.prototype.init = function () {
        var self = this;

        this.$tabs.tabs({
            active: 0,
            create: function (event, ui) {
                self.currentTabIndex = ui.tab.index();
                self.initTabs();
            },
            activate: function (event, ui) {
                currentTabIndex = 0;
                self.currentTabIndex = ui.newTab.index();
                var selections = self.tabs[ui.oldTab.index()].deactivate();
                self.tabs[self.currentTabIndex].activate(selections);
            }
        });

        $(document)
            // Attach event handler to Hypervisor view select box changes
            /*.on("hypervisorviewchange.protect", function (event, views) {
                self.updateTreeView(false, views);
                self.inventoryTree.clearSelections();
            })*/
            .on("changeNavGroups", function (event) {
                self.updateTreeView(false);
                self.inventoryTree.clearSelections();
                self.inventoryTreeHeader.collapseTree();
            })
            .off("failedJobDeleted")
            .on("failedJobDeleted", function (event, failedBids, successCount, warningCount, sid) {
                // Move to ui-grid to re-render column.
                // with ui-grid, we have to reload to re-render the affected rows.
                self.tabs[self.currentTabIndex].loadedInventory.length = 0;
                self.tabs[self.currentTabIndex].update();

                /*event.stopPropagation();
                var failed = $("td").find("[data-failures-ids='" + failedBids + "'][data-sid='" + sid + "']");
                // warning trumps success
                if (warningCount > 0){
                    failed.toggleClass("icon-phd-error icon-phd-warning");
                } else if (successCount > 0){
                    failed.toggleClass("icon-phd-error icon-phd-success");
                } else {
                    failed.toggleClass("icon-phd-error icon-phd-none");
                }*/

            });

        return this;
    };

    ProtectInventoryTab.prototype.updateTreeView = function (views) {
        var self = this,
            load = PHD.showLoadingIndicator($protectTreeContainer),
            hypervisors = views || this.getHypervisorViews(),
            tvTask = PHD.TreeViewBuilder({
                hypervisorView: hypervisors,
                disks: 0,
                checkboxes: false,
                isCopy: 0,
                loader: load,
                uid: $rootScope.user.id     // user id for navigational groups (Protected Assets only, not Copied Assets).
            });

        tvTask
            .done(function (tree) {
                self.TreeViewData = tree.tree;
                $rootScope.treeDataCache = [tree.tree];
                console.log("nav groups: " + $rootScope.showNavGroups);
                if ($rootScope.showNavGroups) {
                    self.navGroups = inventorysvc.buildNavGroups(self.TreeViewData);
                }
                $protectTreeContainer.html(tree.html);

                self.inventoryTreeSearch = PHD.TreeViewSearch(
                    $("#protect-treeview-find"),
                    self.inventoryTree,
                    self.inventoryTree.$container,
                    self.inventoryTree.$container.find(".tree-universe")
                );
                self.tabs.forEach(function (tab) {
                    tab.clear();
                });
            })
            .always(function () {
                PHD.hideLoadingIndicator(load);
            });
    };

    ProtectInventoryTab.prototype.getHypervisorViews = function () {
        return $("#protect-treeview-header").find("select")
            .map(function () {
                return parseInt($(this).val(), 10);
            }).get();
    };

    ProtectInventoryTab.prototype.initTabs = function () {
        var self = this;
        this.$tabs.find(".ui-tabs-panel").each(function (index) {
            self.tabs[index] = new InventoryTab(index, $(this), self.slugs[index], self).init();
        });
    };

    ProtectInventoryTab.prototype.updateButtons = function (tab) {
        var tab = tab || this.tabs[this.currentTabIndex],
            selections = tab.$selected ? tab.$selected.serialize() : null;

        $scope.selectedNode = this.selectedIDs !== undefined ? this.selectedIDs : undefined;
        if ($scope.selectedNode !== undefined) {
            console.log("preselected items");
            console.log($scope.selectedNode);
        }

        this.$buttons.find("a").each(function () {
            var $but = $(this).data("selections", selections);
            PHD.buttonLogic($but, tab.$selected, tab, null, null, {provider: $analytics, source: 'Protect'});
        });
    };

    ProtectInventoryTab.prototype.activate = function () {
        this.active = true;
        this.updateTreeView();

        this.tabs[this.currentTabIndex].activate();
        this.updateButtons();
    };

    ProtectInventoryTab.prototype.deactivate = function () {
        this.active = false;
    };

    ProtectInventoryTab.prototype.setTimerInterval = function (interval) {
        TabTimer.setInterval(refreshRate);
        if (this.$panel.is(":visible")) {
            TabTimer.reset();
        }
    };

    /* Copy Inventory Tab for backup copied assets */
    var CopyInventoryTab = function (index, panel, slug) {
        this.index = index;
        this.$panel = panel;
        this.namespace = "copy" + slug;

        this.$buttons = this.$panel.find(".button-bar");
        this.$buttons.find(".btn-group-recover").flyout();

        this.$tabs = $('#copy-inventory-details');
        this.tabs = [];
        this.slugs = ['status'];

        this.TreeViewData = null;

        this.inventoryTree = PHD.ProtectTreeView(
            $copyTreeContainer,
            $("#copy-treeview-selections")
        );

        this.inventoryTreeHeader = PHD.TreeViewHeader(
            $("#copy-treeview-header"),
            this.inventoryTree,
            "copy"
        );

        this.inventoryTreeSearch = null;
        this.currentTabIndex = 0;
        this.active = false;
    };

    CopyInventoryTab.prototype.init = function () {
        var self = this;

        this.$tabs.tabs({
            active: 0,
            create: function (event, ui) {
                self.currentTabIndex = ui.tab.index();
                self.initTabs();
            },
            activate: function (event, ui) {
                currentTabIndex = 1;
                self.currentTabIndex = ui.newTab.index();
                var selections = self.tabs[ui.oldTab.index()].deactivate();
                self.tabs[self.currentTabIndex].activate(selections);
            }
        });

        return this;
    };

    CopyInventoryTab.prototype.updateTreeView = function (views) {
        var self = this,
            load = PHD.showLoadingIndicator($copyTreeContainer),
            hypervisors = views || this.getHypervisorViews(),
            tvTask = PHD.TreeViewBuilder({
                hypervisorView: hypervisors,
                disks: 0,
                checkboxes: false,
                isCopy: 1,
                loader: load
            });

        tvTask
            .done(function (tree) {
                self.TreeViewData = tree.tree;
                $copyTreeContainer.html(tree.html);

                self.inventoryTreeSearch = PHD.TreeViewSearch(
                    $("#copy-treeview-find"),
                    self.inventoryTree,
                    self.inventoryTree.$container,
                    self.inventoryTree.$container.find(".tree-universe")
                );
                self.tabs.forEach(function (tab) {
                    tab.clear();
                });
            })
            .always(function () {
                PHD.hideLoadingIndicator(load);
            });
    };

    CopyInventoryTab.prototype.getHypervisorViews = function () {
        return $("#copy-treeview-header").find("select")
            .map(function () {
                return parseInt($(this).val(), 10);
            }).get();
    };

    CopyInventoryTab.prototype.initTabs = function () {
        var self = this;
        this.$tabs.find(".ui-tabs-panel").each(function (index) {
            self.tabs[index] = new InventoryTab(index, $(this), self.slugs[index], self).init();
        });
    };

    CopyInventoryTab.prototype.updateButtons = function (tab) {
        var tab = tab || this.tabs[this.currentTabIndex],
            selections = tab.$selected ? tab.$selected.serialize() : null;

        $scope.selectedNode = this.selectedIDs !== undefined ? this.selectedIDs : undefined;
        if ($scope.selectedNode !== undefined) {
            console.log("preselected items");
            console.log($scope.selectedNode);
        }

        this.$buttons.find("a").each(function () {
            var $but = $(this).data("selections", selections);
            PHD.buttonLogic($but, tab.$selected, tab, null, null, {provider: $analytics, source: 'Protect'});
        });
    };

    CopyInventoryTab.prototype.activate = function () {
        this.active = true;
        this.updateTreeView();

        this.tabs[this.currentTabIndex].activate();
        this.updateButtons();
    };

    CopyInventoryTab.prototype.deactivate = function () {
        this.active = false;
    };


    var InventoryTab = function (index, panel, slug, parent) {
        this.index = this.inventoryView = index;
        this.$panel = panel;
        this.namespace = "inventory" + slug;

        this.$table = this.$panel.find("table");

        /*
        * Check to see if this is the Copied Assets Tab or not.
        */
        this.isCopyTab = this.$panel.is("#copy-inventory-status");

        this.schema = this.$table.find("thead th").map(function () {
            return $(this).data("source");
        }).get();
        this.$checkSelectAll = this.$table.find(".check-select-all");

        this.Parent = parent;

        this.$tableMessage = this.$panel.find(".protect-inventory-message");

        this.data = null;

        // set to null, using ui-grid instead.
        this.InventoryTable = null;

        this.$selected = null;
        this.selectedIDs = [];
        this.$selectedInventory = null;
        this.active = false;
        this.timestamp = 0;
    };

    InventoryTab.prototype.init = function () {
        var self = this;

        // Show the general instruction message.
        this.$tableMessage.show();

        this.$table
            .sortTable()
            .filterTable();

        this.Parent.inventoryTree.$container.on("nodeclick", ".checkable", $.proxy(this.handleTreeviewSelection, this));

        // This is the inventory loaded, at the appliance level.
        this.loadedInventory = [];
        this.loadedSystemID = 0;

        return this;
    };

    InventoryTab.prototype.clear = function () {
        this.$selectedInventory = null;
        if (this.InventoryTable) {
            this.InventoryTable.empty();
        }
    };

    InventoryTab.prototype.update = function (selections) {
        if (this.$selectedInventory) {

            // Hide the general instruction message.
            this.$tableMessage.hide();

            var self = this,
                dbid = this.$selectedInventory.data("dbid"),
                type = this.$selectedInventory.data("type");

            var url = "/api/inventory/status";

            if (this.loadedInventory.length === 0 || this.loadedSystemID === 0 ||
                    (this.sid_of(dbid) !== this.loadedSystemID && this.sid_of(dbid) !== -1)) {
                var $element = this.isCopyTab ? $('#copy-inventory-status') : $('#protect-inventory-status');
                var loadHash = PHD.showLoadingIndicator($element); //'body', true, 'loading....'); //TODO this.$table.parent());
                //url += "/?sid=" + this.sid_of(dbid);
                if (this.isCopyTab) {
                    url += "/?grandclient=1";
                } else {
                    var sid = this.sid_of(dbid);
                    if (sid !== -1) {
                        url += "/?sid=" + sid;
                    } else {
                        // it is a group!
                    }
                }
                var resp = PHD.Ajax.get(url, loadHash);
                resp.done(function (data) {
                    PHD.hideLoadingIndicator(loadHash);
                    if (data !== undefined && data.inventory !== undefined) {
                        self.loadedInventory = data.inventory;
                        if (data.inventory.length > 0) {
                            self.loadedSystemID = self.sid_of(data.inventory[0].id);
                        } else {
                            self.loadedSystemID = 0;
                        }
                        self.loadTable(dbid, selections, type);
                    } else {
                        PHD.throwError({"result": [{"message": gettext("An error occurred while loading the status.")}]});
                    }
                });
            } else {
                self.loadTable(dbid, selections, type);
            }
        } else {
            setTimeout(function () {
                if ($scope.gridList[currentTabIndex] !== 0) {
                    $scope.gridApi = $scope.gridList[currentTabIndex];
                }
            }, 0);
            this.$table
                .scrollTable()
                .columnResize();
        }
    };

    InventoryTab.prototype.sid_of = function(dbid) {
        var str_dbid = String(dbid);
        var a = str_dbid.split('_');
        var sid = a[0];
        if (isNaN(sid)) {
            sid = -1;
        } else {
            sid = parseInt(sid, 10);
        }
        return sid;
    };


    InventoryTab.prototype.loadTable = function (dbid, selections, nodeType) {

        var str_dib = String(dbid);
        var data = [];
        data.inventory = [];

        function isAChild(obj, item_id, data) {
            if (obj.id === item_id) {
                data.inventory.push(obj);
            } else {
                var nodeLength = item_id.length;
                if ((obj.id.substr(0, nodeLength) === item_id) && (obj.id.charAt(nodeLength) === '_')) {
                    data.inventory.push(obj);
                }
            }
        }

        // Filter to the item selected.
        $scope.selectedInventoryID = str_dib;
        var isGroupSelected = nodeType === INVENTORY_ID_NAVGROUP; // was inventorysvc.isGroup(str_dib);
        var isPoolOrvAppSelected = nodeType === INVENTORY_ID_VMWARE_RESOURCE_POOL || nodeType === INVENTORY_ID_VMWARE_V_APP;
        if (!isPoolOrvAppSelected) {
            for (var i = 0; i < this.loadedInventory.length; i++) {
                var obj = this.loadedInventory[i];
                if (obj.id !== undefined) {
                    if (!isGroupSelected) {
                        isAChild(obj, str_dib, data);
                    } else {
                        if (inventorysvc.isGroupMember(dbid, obj.id, this.Parent.navGroups)) {
                            data.inventory.push(obj);
                        }
                    }
                }
            }
        } else {
            // Handle pools by looking at their descendants in the tree; you cannot tell by the ID pieces only.
            var matches = this.Parent.inventoryTree.$container.find("[data-dbid=" + str_dib + "] .children .checkable");
            if (matches !== undefined) {
                for (var j = 0; j < matches.length; j++) {
                    var containerID = matches[j].dataset.dbid;
                    for (i = 0; i < this.loadedInventory.length; i++) {
                        obj = this.loadedInventory[i];
                        if (obj.id !== undefined) {
                            isAChild(obj, containerID, data);
                        }
                    }
                }
            }
        }

        if (this.InventoryTable) {
            this.InventoryTable.update(data.inventory, true);
        } else {
            setTimeout(function () {
                $scope.updateGridData(currentTabIndex, data.inventory);
                $scope.$apply();
            }, 0);
        }

        if (selections) {
            this.selectedIDs = selections;
            this.$table.find("[name=inventory]").prop("checked", false);
            this.selectedIDs.forEach(function (value) {
                self.$table.find("[value=" + value + "]").prop("checked", true);
            });
            this.updateSelections();
        }

        this.$table
            //.sortTable() -- no sort here, will sort based on datatable updatefinished event.
            .scrollTable()
            .columnResize();

        this.Parent.updateButtons(this);
    };

    InventoryTab.prototype.updateSelections = function () {
        this.$selected = this.$table.find("[name=inventory]:checked");
        this.selectedIDs = this.$table.find("tbody :checked").map(function () {
            return $(this).val();
        }).get();
        this.Parent.updateButtons(this);
    };

    InventoryTab.prototype.handleTreeviewSelection = function (event, id) {
        event.stopPropagation();
        this.$selectedInventory = $(event.target);
        if (this.active) {
            this.update();
        }
    };

    InventoryTab.prototype.activate = function (selections) {
        this.active = true;

        var tabIndex = currentTabIndex;
        if ($scope.gridList[tabIndex] !== 0) {
            $scope.gridApi = $scope.gridList[tabIndex];
            $scope.columnOrderRestored = false;
        }

        this.update(selections);
    };

    InventoryTab.prototype.deactivate = function () {
        this.active = false;
        return this.selectedIDs;
    };

    $(document)
        .on("uinav.dashboard", stopAll)
        .on("uinav.jobs", stopAll)
        .on("uinav.recover", stopAll)
        .on("uinav.configure", stopAll)
        .on("uinav.reports", stopAll)
        .on("userpreferencesave.protectpage", function (event, key, value) {
            setPageRefresh(value);
        })
        .on("timersetmanual.protectpage", function (event, $control) {
            $control.show();
        })
        .on("timerunsetmanual.protectpage", function (event, $control) {
            $control.hide();
        });

    function stopAll(event) {
        console.log("===>In Stop All: Protect Tab, event " + event.namespace);
        TabTimer.stop();
        UserPreference.cancel();
        $(document).off("uinav.dashboard", stopAll);
        $(document).off("uinav.jobs", stopAll);
        $(document).off("uinav.recover", stopAll);
        $(document).off("uinav.configure", stopAll);
        $(document).off("uinav.reports", stopAll);
        $(document).off("userpreferencesave.protectpage");
        $(document).off("timertick.protectpage");
        $(document).off("timersetmanual.protectpage");
        $(document).off("timerunsetmanual.protectpage");
        $(document).off("changeNavGroups");
        $(document).off("failedJobDeleted");
    }

    function handleTabChange() {
        $(document).trigger("tabchange", ["protect" + slugs[currentTabIndex]]);
    }

    /*
    *   DOM Ready Function
    */
    $(function () {
        // Quick and dirty way to change what tab opens by default via URL
        if (!isNaN(PHD.util.getHashInt())) {
            initialTab = PHD.util.getHashInt();
        }

        UserPreference = PHD.UserPreference($("#protect-page-preferences .preference-trigger"), {
            name: gettext("Protect Page"),
            namespace: "protectpage",
            key: "page_protect_refresh",
            defaultValue: refreshRate
        });

        /*TabTimer = PHD.Timer({
            namespace: "protectpage",
            interval: refreshRate,
            refreshControl: $("#protect-page-preferences .timer-refresh")
        });
        */

        $('.tabs-wrapper').tabs({
            active: initialTab,
            create: function (event, ui) {
                currentTabIndex = ui.tab.index();
                initTabs();
                handleTabChange();
            },
            activate: function (event, ui) {
                currentTabIndex = ui.newTab.index();
                $.cookie("initial_tab_protect", currentTabIndex, {expires: 365, path: '/'});
                tabs[ui.oldTab.index()].deactivate();
                tabs[currentTabIndex].activate();
                handleTabChange();
            }
        });
    });

    var dayTmpl = _.template(
        '<p><label class="day-status-day"><span><%= dayString %></span></label></p>' +
        '<p><label><translate><%= type %></translate></label></p>' +
        '<hr><label><translate>Successes</translate>:</label><span><%= successes.count %></span>' +
        '<br/><label><translate>Warnings</translate>:</label><span><%= warnings.count %></span>' +
        '<br/><label><translate>Failures</translate>:</label><span><%= failures.count %></span>' +
        '<br/><label><translate>In Progress</translate>:</label><span><%= incomplete.count %></span>' +
        '<hr>' +
        '<p><a href="" ng-click="loadStatus(<%= sid %>, \'<%= all_ids %>\', \'<%= formatted_day %>\', \'<%= formatted_name %>\', \'<%= type %>\', <%= override_copies %>)" ' +
        'class="job-log-link"><i class="icon-list-alt"></i><translate>Details</translate></a></p>'
    );

    var targetsTmpl = _.template(

        '<p><label class="day-status-day"><span><%= dayString %></span></label></p>' +
        '<p><label><translate><%= type %></translate></label></p>' +

        '<% for (var i = 0; i < targets.length; i++) { %>' +
        '<% var target = targets[i]; %>' +
            '<hr><label><translate>Target</translate>:</label><span><%= target.name %></span>' +
            '<br/><label><translate>Successes</translate>:</label><span><%= target.successes.count %></span>' +
            '<br/><label><translate>Failures</translate></label><span><%= target.failures.count %></span>' +
            '<p><a href="" ng-click="loadStatus(<%= sid %>, \'<%= target.ids %>\', \'<%= formatted_day %>\', \'<%= formatted_name %>\', \'<%= type %>\')" ' +
                'class="job-log-link"><i class="icon-list-alt"></i><translate>Details</translate></a></p>' +
            '<br/>' +
        '<% } %>'
    );
}
