angular.module('manage-backups-module', ['ngDialog', 'ui.grid','ui.grid.resizeColumns','ui.grid.autoResize','ngGrid','ui.grid.selection', 'ui.grid.pagination'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('manageBackupsCtrl', ['$scope', '$rootScope', '$http','ngDialog', '$window', '$timeout', '$compile', 'uiGridConstants', 'gettextCatalog',
        function($scope, $rootScope, $http, ngDialog, $window, $timeout, $compile, uiGridConstants, gettextCatalog) {
            $scope.backupBrowserData = $scope.backupsData;

            $scope.gridData = [$scope.backupsData];
            $scope.selectedItem = null;
            $scope.savedItem = null;  // save item to restore on cancel
            $scope.refresh = false;
            $scope.validBackups = false;
            $scope.invalidBackups = false;
            $scope.deleteBackups_valid = [];
            $scope.deleteBackups_invalid = [];
            $scope.backupsToHold = [];
            $scope.applianceList = [];
            $scope.selected = null;
            $scope.dateRange = {};
            $scope.startDate = null;
            $scope.endDate = null;
            $scope.sid = $rootScope.local.id;
            $scope.holdButton = "Hold";
            $scope.holdDisabled = $rootScope.configureDisabled;        // Disable hold if an operator.
            $scope.restoreFilesDisabled = false;
            $scope.restoreDisabled = false;

            var selectedBackup = {};

        $http({
                method: 'get',
                url : '/api/systems'
            }).success(function(data) {
                for (var i = 0; i < data.appliance.length; i++) {
                    //Don't try to add a client to a system that is not available
                    if (data.appliance[i].status !== "not available") {
                        var obj = {
                            id: data.appliance[i].id,
                            name: data.appliance[i].name
                        };
                        $scope.applianceList.push(obj);
                    }
                }
                for (var i = 0; i < $scope.applianceList.length; i++) {
                    if ($scope.applianceList[i].id === $rootScope.local.id) {
                        $scope.selected = {};
                        $scope.selected = $scope.applianceList[i];
                    }
                }
            });

            $scope.reloadGrid = function () {
                var load = PHD.showLoadingIndicator('body', true, "Refreshing grid...");
                var startDate = DateTimeHandler.getUSDateString($rootScope.userLanguageCode, $scope.startDate);
                var endDate = DateTimeHandler.getUSDateString($rootScope.userLanguageCode, $scope.endDate);
                var url = "/api/backups/browser/?sid=" + $scope.sid + "&start_date=" + startDate + "&end_date=" + endDate +
                            "&lang=" + $rootScope.userLanguageCode;

                $http({
                    method: 'get',
                    url: url
                }).success(function (data, status, headers) {

                    PHD.hideLoadingIndicator(load);
                    $scope.selectedItem = null;
                    $scope.savedItem = null;
                    $scope.holdButton = "Hold";

                    $timeout(function () {
                        $scope.gridApi.selection.clearSelectedRows();
                        if (data.BackupStatus) {
                            $scope.backupsData = data.BackupStatus;
                        } else {
                            $scope.backupsData = [];
                        }
                        $scope.$apply()
                    }, 0);
                }).error(function (response) {

                });
            };

            $scope.manageBackupsData = {
                data: 'backupsData',
                multiSelect: true,
                selectionRowHeaderWidth: 30,
                rowHeight: 26,
                enableFullRowSelection : true,
                enableFiltering: true,
                enableSorting: true,
                enableColumnResizing: true,
                gridMenuShowHideColumns:true,
                enableColumnMenus:true,
                enableGridMenu: true,
                paginationPageSizes: [10,25, 50, 100],
                paginationPageSize: 25,
                enablePaginationControls: true,
                totalItems: $scope.backupsData.length,
                appScopeProvider: $scope.uiGridProvider,
                showGridFooter: true,
                enableHorizontalScrollbar: 0,
                onRegisterApi: function (gridApi) {
                    $scope.onRegisterGrid(gridApi);
                }
            };

            $scope.onRegisterGrid = function (gridApi) {
                $scope.gridApi = gridApi;

                gridApi.selection.on.rowSelectionChanged($scope, $scope.onSelectionChanged);
                gridApi.selection.on.rowSelectionChangedBatch($scope, $scope.onSelectionChanged);

                //default date range to last 24 hours
                var elem = angular.element(document.querySelector('[ng-controller=DateRangeCtrl]'));
                $scope.dateScope = elem.scope();
                $scope.dateScope.setDateRange(0, 86400);

                $scope.dateRange = $scope.dateScope.getDateRange();
                if ($scope.dateRange !== undefined) {
                    $scope.startDate = moment($scope.dateRange.startDate).format(DATE_DISPLAY_FORMAT);
                    $scope.endDate = moment($scope.dateRange.endDate).format(DATE_DISPLAY_FORMAT);
                }
            };

            $scope.onSelectionChanged = function(rows) {
                var gridApi = $scope.gridApi;
                var selectedItems = gridApi.selection.getSelectedRows();
                if (selectedItems.length > 0) {
                    $scope.selectedItem = selectedItems[0];
                } else if (selectedItems.length == 0) {
                    $scope.selectedItem = null;
                }
                $scope.handleButtons(selectedItems);
            };

            $scope.manageBackupsData.columnDefs = [
                {
                    field: "system_id",
                    visible: false
                },
                {
                    field: "app_name",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Application"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "asset_name",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Protected Asset"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "vm_name",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("VM"),
                    cellTooltip: true,
                    visible: false,
                    type: "string"
                },
                {
                    field: "database_name",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Database "),
                    cellTooltip: true,
                    visible: false,
                    type: "string"
                },
                {
                    field: "server_instance_name",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Instance Name"),
                    cellTooltip: true,
                    visible: false,
                    type: "string"
                },
                {
                    field: "id",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Backup ID"),
                    cellTooltip: true,
                    visible: true,
                    type: "number"
                },
                {
                    field: "complete",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Complete"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "status",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Status"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "start_time",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Start Date"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "complete_time",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("End Date"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "duration",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Duration"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },

                {
                    field: "type",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Type"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "size",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Size (MB)"),
                    cellTooltip: true,
                    visible: false,
                    type: "number"
                },
                {
                    field: "at_risk",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("At Risk"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "legalhold",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Hold"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "encrypted",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Encrypted"),
                    cellTooltip: true,
                    visible: false,
                    type: "string"
                },
                {
                    field: "gfs_rp_type",
                    headerCellFilter: 'translate',
                    headerTooltip: gettextCatalog.getString("Available only if the selected appliance's strategy is LTR."),
                    displayName: gettextCatalog.getString("LTR Points"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "storage",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Storage"),
                    cellTooltip: true,
                    visible: false,
                    type: "string"
                },
                {
                    field: "appliance",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Appliance"),
                    cellTooltip: true,
                    visible: false,
                    type: "string"
                }
            ];



            $scope.onChangeAppliance = function (sid){
                $scope.sid = sid;
                $scope.reloadGrid();
            };

            $scope.onChangeDateRange = function(selectedDateRange){
                $scope.dateScope.selectedDateRange = selectedDateRange;
                $scope.dateScope.onDateRangeChange(selectedDateRange);
                $scope.dateRange = $scope.dateScope.getDateRange();
                if($scope.dateRange !== undefined){
                    $scope.startDate = moment($scope.dateRange.startDate).format(DATE_DISPLAY_FORMAT);
                    $scope.endDate = moment($scope.dateRange.endDate).format(DATE_DISPLAY_FORMAT);

                    $scope.reloadGrid();
                }
            };

            $scope.handleButtons = function(selectedItems){
                /**
                    if all selected are same hold status, set button label accordingly
                    if selected are not same hold status, disable button
                **/
                if (selectedItems) {
                    var unhold = false, hold = false;
                    $scope.restoreFilesDisabled = false;
                    $scope.restoreDisabled = false;
                    $scope.holdDisabled = $rootScope.configureDisabled;

                    for (var i = 0; i < selectedItems.length; i++) {
                        if (selectedItems[i].legalhold && !hold) {
                            $scope.holdButton = "Unhold";
                            unhold = true;
                        } else if (!selectedItems[i].legalhold && !unhold) {
                            $scope.holdButton = "Hold";
                            hold = true;
                        } else {
                            $scope.holdDisabled = true;
                        }

                        var appType = selectedItems[i].app_type;
                        switch (appType) {
                            case APPLICATION_TYPE_NAME_SQL_SERVER:
                            case APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE:
                            case APPLICATION_TYPE_NAME_NDMP_DEVICE:
                                $scope.restoreFilesDisabled = true;
                                break;
                        }

                        // don't allow restore of block-level backups
                        var appID = selectedItems[i].app_id;
                        switch (appID) {
                            case APP_ID_BLOCK_LEVEL:
                                $scope.restoreDisabled = true;
                                break;
                        }

                        //allow attempt to restore files from failed file level backup
                        //don't allow hold of failed backups
                        if (selectedItems[i].status === "Failed"){
                            $scope.holdDisabled = true;
                            if(appType !== APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
                                $scope.restoreFilesDisabled = true;
                                $scope.restoreDisabled = true;
                            }
                        }

                        //don't allow imported replicated backups to be held
                        if (selectedItems[i].replicated){
                            $scope.holdDisabled = true;
                        }
                    }
                    //only allow one selected item for restore/restore files for now.  mult-sql will be implemented later
                    if(selectedItems.length > 1){
                        $scope.restoreFilesDisabled = true;
                        $scope.restoreDisabled = true;
                    }
                }
            };

            $scope.deleteBackups = function () {
                $scope.deleteBackups_valid.length = 0;
                $scope.deleteBackups_invalid.length = 0;
                var selectedBackupsToDelete = $scope.gridApi.selection.getSelectedRows();
                if (selectedBackupsToDelete.length > 0) {
                //  $scope.validBackups = true;
                    var selectedSystemIDs = [];

                    var delimiter = ",";
                    var backupIDs = "";
                    for (var i = 0; i < selectedBackupsToDelete.length; i++) {
                        backupIDs += selectedBackupsToDelete[i].id + delimiter;
                        selectedSystemIDs.push(selectedBackupsToDelete[i].system_id);
                    }
                }

                function sameSystemIDsSelected(currentID) {
                    return currentID === selectedSystemIDs[0];
                }

                if (!selectedSystemIDs.every(sameSystemIDsSelected)) {
                    ngDialog.open({
                        dialogType: 'retry',
                        dialogMessage: 'Backups to act on must be from the same appliance.'
                    })
                } else {

                    var load = PHD.showLoadingIndicator('body', true, "Gathering related backups...");
                    sid = selectedSystemIDs[0];

                    //trim the last ',' from the backupIDs string
                    backupIDs = backupIDs.substring(0, backupIDs.length - 1);
                    var bidArray = backupIDs.split(',');
                    var payload = JSON.stringify(bidArray);
                    var url = '/api/backups/?sid=' + sid + '&original&isRelated';

                    var resp = PHD.Ajax.post(url, payload);
                    resp.done(function (data) {
                        PHD.hideLoadingIndicator(load);
                        if (data && data.data.length > 0) {
                            var backupsToDelete = data.data;
                            for (i = 0; i < backupsToDelete.length; i++) {
                                var validObj = {};
                                var invalidObj = {};

                                if (backupsToDelete[i].legalhold) {
                                    invalidObj.backup_id = backupsToDelete[i].id;
                                    invalidObj.asset_name = backupsToDelete[i].asset_name;
                                    invalidObj.type = backupsToDelete[i].type;
                                    invalidObj.date = backupsToDelete[i].start_time;
                                    invalidObj.status = backupsToDelete[i].status;
                                    invalidObj.gfs_rp_type = backupsToDelete[i].gfs_rp_type;
                                    invalidObj.legalhold = backupsToDelete[i].legalhold;

                                    $scope.deleteBackups_invalid.push(invalidObj);

                                } else {
                                    validObj.backup_id = backupsToDelete[i].id;
                                    validObj.asset_name = backupsToDelete[i].asset_name;
                                    validObj.type = backupsToDelete[i].type;
                                    validObj.date = backupsToDelete[i].start_time;
                                    validObj.status = backupsToDelete[i].status;
                                    validObj.gfs_rp_type = backupsToDelete[i].gfs_rp_type;

                                    $scope.deleteBackups_valid.push(validObj);
                                }
                            }

                            $scope.validBackups = $scope.deleteBackups_valid.length > 0;
                            $scope.invalidBackups = $scope.deleteBackups_invalid.length > 0;

                            var actionsStyle = 'width:900px; height:' + ($scope.invalidBackups && $scope.validBackups ? '525' : '325') + 'px;';

                            ngDialog.open({
                                template: 'app/recover/manage-backups/manageBackups-actions.html',
                                scope: $scope,
                                overlay: true,
                                modelDialogId: 'manage-backups-actions-dialog',
                                ngDialogStyle: actionsStyle,
                                closeByDocument: false,
                                closeByEscape: true
                            });
                        }
                    }).fail(function(){
                        PHD.hideLoadingIndicator(load);
                    });
                }
            };

            $scope.doDelete = function () {
                var ids = _.pluck($scope.deleteBackups_valid, 'backup_id');
                var payload = JSON.stringify(ids);
                var url = "/api/backups/" + "?sid=" + sid + "&isDelete";
                var resp = PHD.Ajax.post(url, payload);
                resp.done(function(data) {
                    $scope.reloadGrid();
                    ngDialog.close("manage-backups-actions-dialog");

                });
            };

            $scope.confirmHold = function () {
                $scope.backupsToHold.length = 0;
                var selectedBackupsToHold = $scope.gridApi.selection.getSelectedRows();
                if (selectedBackupsToHold.length > 0) {
                    for (var n = 0; n < selectedBackupsToHold.length; n++) {
                        var backupID = selectedBackupsToHold[n].id;
                        sid = selectedBackupsToHold[n].system_id;
                        //if the backup is on hold, we're taking it off legal hold
                        var hold = selectedBackupsToHold[n].legalhold ?  0 : -1;
                        var url = "/api/backups/hold/" + backupID + "/" + hold + "/?sid=" + sid;
                        var resp = PHD.Ajax.put(url);
                        resp.done(function (data) {
                        });
                    }
                    $scope.reloadGrid();
                }
            };

            $scope.confirmRestore = function () {
                $('#restoreBtn').removeClass("disabled");
                $scope.openRecoverDialog($scope.selectedItem);
            };
            $scope.confirmRestoreFiles = function () {
                $scope.handleButtons($scope.selectedItems);
                $('#restoreFilesBtn').removeClass("disabled");
                $scope.openFileLevelRecoveryJobDialogClick($scope.selectedItem.id, $scope.selectedItem.app_type, $scope.selectedItem.app_id, true);
            };

            $scope.validDeleteBackupsGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 24,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                data: $scope.deleteBackups_valid
            };

            $scope.validDeleteBackupsGridData.columnDefs = [
                {
                    field: "asset_name",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Protected Asset"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },

                {
                    field: "backup_id",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Backup ID"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "gfs_rp_type",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("LTR Points"),
                    headerTooltip: gettextCatalog.getString("Available only if the selected appliance's strategy is LTR."),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "type",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Type"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "date",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Date"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                },
                {
                    field: "status",
                    headerCellFilter: 'translate',
                    displayName: gettextCatalog.getString("Status"),
                    cellTooltip: true,
                    visible: true,
                    type: "string"
                }
            ];

            $scope.invalidDeleteBackupsGridData = {
                multiSelect: false,
                selectionRowHeaderWidth: 35,
                rowHeight: 24,
                enableFullRowSelection : true,
                gridMenuShowHideColumns:false,
                enableColumnMenus:false,
                enableHorizontalScrollbar: 0,
                data: $scope.deleteBackups_invalid
            };

            $scope.invalidDeleteBackupsGridData.columnDefs = $scope.validDeleteBackupsGridData.columnDefs;

            $scope.closeThisDialog = function() {
                $scope.deleteBackups_valid.length = 0;
                $scope.deleteBackups_invalid.length = 0;

                ngDialog.close("manage-backups-actions-dialog");
            };

        }]);
